package Net::Patricia::ipvXbycc;

=head1 NAME

Net::Patricia::ipvXbycc - Net::Patricia & http://nami.jp/ipv4bycc/ & https://foxmaster.jp/ipv6bycc/

=head1 SYNOPSIS

    use Net::Patricia::ipvXbycc;

    my $ipbycc = Net::Patricia::ipvXbycc->new;
    say $ipbycc->match_string('8.8.8.8');              # US
    say $ipbycc->match_string('2001:4860:4860::8888')  # US

    # If you've had the instance for a long time and you want to check the mtime of the db file.
    $ipbycc->delete_pt_if_db_mtime_is_old;
        OR
    my $ipbycc = Net::Patricia::ipvXbycc->new( auto_mtime_check_sec => 10 );

    __END__

    % crontab -e

    0 5 * * *  update_ipvXbycc_db.pl --verbose

=cut

use strict;
use warnings;
use File::Share qw(dist_dir dist_file);
use Net::Patricia;
use Socket qw(AF_INET AF_INET6);

sub new {
    my $class = shift;
    bless { @_ }, $class;
}

sub db_file {
    my ($self, $type) = @_;
    die unless defined $type and ( $type == AF_INET or $type == AF_INET6 );
    my $v = $type == AF_INET ? 4 : 6;
    my $cidr_file = dist_file 'Net-Patricia-ipvXbycc', "ipv${v}bycc.cidr.txt";
}

sub delete_pt_if_db_mtime_is_old {
    my ($self, $t) = @_;

    my $num_delete = 0;

    for my $type (AF_INET, AF_INET6) {
        next unless exists $self->{"pt$type"};
        next unless !defined $t or $t == $type;
        my $db_file = $self->db_file($type);
        my $mtime1  = $self->{"pt${type}_mtime"};
        my $mtime2  = (stat($db_file))[9];
        if ( $mtime1 < $mtime2 ) {
            delete $self->{"pt${type}"};
            delete $self->{"pt${type}_mtime"};
            delete $self->{"pt${type}_check"};
            $num_delete++;
        }
        else {
            $self->{"pt${type}_check"} = time;
        }
    }

    $num_delete;
}

sub pt {
    my ($self, $type) = @_;
    die unless defined $type and ( $type == AF_INET or $type == AF_INET6 );

    if ( my $sec = $self->{auto_mtime_check_sec} ) {
        if ( my $check = $self->{"pt${type}_check"} ) {
            if ( ($check+$sec) < time ) {
                $self->delete_pt_if_db_mtime_is_old($type);
            }
        }
    }

    unless ( exists $self->{"pt$type"} ) {
        my $pt = Net::Patricia->new($type);
        my $cidr_file = $self->db_file($type);
        open my $in, "<", $cidr_file or die;
        local $_;
        while (<$in>) {
            if ( /^(\S\S)\t(\S+)$/ ) {
                $pt->add_string($2, $1);
            }
        }
        $self->{"pt${type}"}       = $pt;
        $self->{"pt${type}_mtime"} = (stat($in))[9];
        $self->{"pt${type}_check"} = time;
    }

    $self->{"pt$type"};
}

{
    my @methods = qw(add_string match_string match_exact_string remove_string);
    for my $meth (@methods) {
        my $code = sub {
            my ($self, $ip) = (shift, @_);
            my $type = index($ip, ":") == -1 ? AF_INET : AF_INET6;
            my $pt   = $self->pt($type);
            $pt->$meth(@_);
        };
        no strict 'refs';
        *$meth = $code;
    }
}

1;
